package cc.fascinated.services;

import cc.fascinated.exception.impl.BadRequestException;
import cc.fascinated.model.score.TotalScoresMetric;
import cc.fascinated.model.score.TrackedScore;
import cc.fascinated.model.score.TrackedScoreDTO;
import cc.fascinated.platform.Platform;
import cc.fascinated.repository.couchdb.TrackedScoreRepository;
import lombok.NonNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Service
public class TrackedScoreService {
    /**
     * The tracked score repository to use.
     */
    @NonNull private final TrackedScoreRepository trackedScoreRepository;

    @Autowired
    public TrackedScoreService(@NonNull TrackedScoreRepository trackedScoreRepository) {
        this.trackedScoreRepository = trackedScoreRepository;
    }

    /**
     * Gets a list of top tracked scores
     * sorted by pp from the platform
     *
     * @param platform the platform to get the scores from
     * @param amount the amount of scores to get
     * @return the scores
     */
    public List<TrackedScoreDTO> getTopScores(Platform.Platforms platform, int amount) {
        List<TrackedScore> scores = trackedScoreRepository.findTopRankedScores(platform.getPlatformName(), amount);
        if (scores.isEmpty()) {
            throw new BadRequestException("No scores found for platform " + platform.getPlatformName());
        }
        return scores.stream().map(TrackedScore::getAsDTO).toList();
    }

    /**
     * Gets the total amount of scores
     * for a platform.
     *
     * @param platform the platform to get the scores from
     * @return the total amount of scores for the platform
     */
    public TotalScoresMetric getTotalScores(Platform.Platforms platform) {
        return new TotalScoresMetric(
                trackedScoreRepository.countTotalScores(platform.getPlatformName()),
                trackedScoreRepository.countTotalRankedScores(platform.getPlatformName())
        );
    }
}
